#ifndef SOLVER_HH
#define SOLVER_HH

#include "function.hh"


/**
 * Interface for representing solvers for equations
 *   r(z) = 0,
 * with r: R^n --> R^n differentiable, n \in N.
 * @tparam Vector The type representing a vector in R^n.
 * @tparam Matrix The type representing a matrix in R^{n \times n},
 *                e.g. jacobian matrices of r.
 */
template <typename Vector, typename Matrix>
class Solver
{
public:
  using VectorType = Vector;
  using MatrixType = Matrix;
  using FunctionType = DifferentiableFunction<VectorType,VectorType,MatrixType>;

  /**
   * Pure virtual function (abstract method) that applies the solver,
   * i.e. solves the equation.
   * @param[in]     r Function which defines the equation (residual function).
   * @param[in,out] z Solution of the equation;
   *                  contains initial guess e.g. for iterative solvers.
   */
  virtual void apply (const FunctionType& r, VectorType& z) const = 0;
};

#endif // SOLVER_HH
